import json

import boto3
from requests_aws4auth import AWS4Auth

session = boto3.Session()
lambda_client = boto3.client("lambda")


def get_aws_auth():
    aws_auth = AWS4Auth(
        region=session.region_name,
        service="lambda",
        refreshable_credentials=session.get_credentials(),
    )
    return aws_auth


def get_base_url(key: str):
    return f"https://{key}.lambda-url.{session.region_name}.on.aws/"


def get_user():
    profile = session.profile_name
    profiles = session._session.full_config["profiles"]
    if profile in profiles:
        config = session._session.full_config["profiles"][session.profile_name]
        if "source_profile" in config:
            profile = config["source_profile"]
    user = (
        boto3.Session(profile_name=profile).client("sts").get_caller_identity()["Arn"]
    )
    return user


def set_limit(
    limit: float, staging: str, project: str, model: str = "*", user: str = "*"
):
    payload = {
        "project": project,
        "user": user,
        "model": model,
        "limit": limit,
    }
    result = json.loads(
        lambda_client.invoke(
            FunctionName=f"openai-admin-{staging}",
            InvocationType="RequestResponse",
            Payload=json.dumps(payload),
        )["Payload"].read()
    )
    if "errorMessage" in result:
        raise Exception(result["errorMessage"])


def flush_cache(staging: str):
    payload = {
        "flush_cache": True,
    }
    result = json.loads(
        lambda_client.invoke(
            FunctionName=f"openai-admin-{staging}",
            InvocationType="RequestResponse",
            Payload=json.dumps(payload),
        )["Payload"].read()
    )
    if "errorMessage" in result:
        raise Exception(result["errorMessage"])



import os
from typing import Any, TypeVar, Union

import httpx
from httpx import URL
from openai._base_client import BaseClient
from openai._client import AsyncOpenAI, OpenAI
from openai._streaming import AsyncStream, Stream

from .utils import get_aws_auth, get_base_url, get_user

_HttpxClientT = TypeVar("_HttpxClientT", bound=Union[httpx.Client, httpx.AsyncClient])
_DefaultStreamT = TypeVar("_DefaultStreamT", bound=Union[Stream[Any], AsyncStream[Any]])


class BaseClientProxy(BaseClient[_HttpxClientT, _DefaultStreamT]):
    custom_auth = get_aws_auth()


class Proxy:
    def __init__(
        self,
        project: str = os.environ.get("OPENAI_DEFAULT_PROJECT", "N/A"),
        staging: str = os.environ.get("OPENAI_DEFAULT_STAGING", "dev"),
        caching: str = os.environ.get("OPENAI_DEFAULT_CACHING", "1"),
    ):
        self.set_project(project)
        self.set_staging(staging)
        self.set_caching(caching)
        self._custom_headers["openai-proxy-user"] = get_user()
        self._base_url = URL(get_base_url(self.api_key.split("-")[1]))

    def set_project(self, project: str):
        self._custom_headers["openai-proxy-project"] = project

    def set_staging(self, staging: str):
        self._custom_headers["openai-proxy-staging"] = staging

    def set_caching(self, caching: bool):
        self._custom_headers["openai-proxy-caching"] = str(int(caching))


class OpenAIProxy(BaseClientProxy[httpx.Client, Stream[Any]], OpenAI, Proxy):
    def __init__(self, **kwargs):
        proxy_kwargs = {
            k: kwargs.pop(k)
            for k in list(kwargs)
            if k in ["project", "staging", "caching"]
        }
        OpenAI.__init__(self, **kwargs)
        Proxy.__init__(self, **proxy_kwargs)


class AsyncOpenAIProxy(BaseClientProxy[httpx.Client, Stream[Any]], AsyncOpenAI, Proxy):
    def __init__(self, **kwargs):
        proxy_kwargs = {
            k: kwargs.pop(k)
            for k in list(kwargs)
            if k in ["project", "staging", "caching"]
        }
        AsyncOpenAI.__init__(self, **kwargs)
        Proxy.__init__(self, **proxy_kwargs)



import os
from types import SimpleNamespace
from typing import Dict, Optional, Tuple
from urllib.parse import urljoin

import openai
from openai import api_key

from .utils import get_aws_auth, get_base_url, get_user

_base_url = get_base_url(api_key.split("-")[1])
_custom_auth = get_aws_auth()
_custom_headers = {
    "openai-proxy-user": get_user(),
    "openai-proxy-project": os.environ.get("OPENAI_DEFAULT_PROJECT", "N/A"),
    "openai-proxy-staging": os.environ.get("OPENAI_DEFAULT_STAGING", "dev"),
    "openai-proxy-caching": os.environ.get("OPENAI_DEFAULT_CACHING", "1"),
}


def set_project(project: str):
    _custom_headers["openai-proxy-project"] = project


def set_staging(staging: str):
    _custom_headers["openai-proxy-staging"] = staging


def set_caching(caching: bool):
    _custom_headers["openai-proxy-caching"] = str(int(caching))


_prepare_request_raw = openai.api_requestor.APIRequestor._prepare_request_raw


def _prepare_request_raw_proxy(
    self,
    url,
    supplied_headers,
    method,
    params,
    files,
    request_id: Optional[str],
) -> Tuple[str, Dict[str, str], Optional[bytes]]:
    _, headers, data = _prepare_request_raw(
        self, url, supplied_headers, method, params, files, request_id
    )

    request = _custom_auth(
        SimpleNamespace(
            **{
                "method": method,
                "url": urljoin(_base_url, url),
                "headers": {**headers, **_custom_headers},
                "content": data,
            }
        )
    )
    return request.url, request.headers, request.content


# Monkey patch
openai.api_requestor.APIRequestor._prepare_request_raw = _prepare_request_raw_proxy



import importlib
import sys

from packaging import version

from .utils import flush_cache, set_limit  # noqa: F401

openai_orig = importlib.import_module("openai")
parsed_version = version.parse(openai_orig.__version__)
major = parsed_version.major
assert (
    parsed_version.major == 0 and parsed_version >= version.parse("0.28.1")
) or parsed_version >= version.parse("1.2.3")


if major == 0:
    from .proxy0 import set_caching, set_project, set_staging  # noqa: F401

    globals().update(vars(openai_orig))
    sys.modules["openai"] = sys.modules["openai_wrapi"]

else:
    from .proxy1 import AsyncOpenAIProxy, OpenAIProxy

    globals().update(vars(openai_orig))
    sys.modules["openai"] = sys.modules["openai_wrapi"]
    client = OpenAIProxy()
    for attr in dir(client):
        if not attr.startswith("__"):
            globals()[attr] = getattr(client, attr)
    Client = OpenAI = OpenAIProxy
    AsyncOpenAI = AsyncOpenAIProxy

sys.modules["openai"] = sys.modules["openai_wrapi"]
